/*

 I'm confused here.

 poll() often gets EINTR. So we can expect the same to happen to
 read() and write().

 I haven't found WHO sends WHICH signal. Is it the pthreads ?
 Is it sleep() ? Is it gdb ? Is it gdb/threads ?

 EINTR is useful for program termination by signal.
 It's just raised occasionally by "somebody".

 I'm assuming that threads cause EINTR to poll.

 So here we have a global variable "EINTR_Valid"
 These wrappers to poll(), read(), write() will ingore
 EINTR errors as long as EINTR_Valid is not set.

*/

#include <sys/poll.h>
#include <unistd.h>
#include <errno.h>

int EINTR_Valid = 0;

size_t p_read (int fd, void *buf, size_t count)
{
	size_t s;

	do s = read (fd, buf, count);
	while (s == -1 && errno == EINTR && !EINTR_Valid);

	return s;
}

size_t p_write (int fd, void *buf, size_t count)
{
	size_t s;

	do s = write (fd, buf, count);
	while (s == -1 && errno == EINTR && !EINTR_Valid);

	return s;
}

int p_poll (struct pollfd *ufds, unsigned int nfds, int timeout)
{
	int r;

	do r = poll (ufds, nfds, timeout);
	while (r == -1 && errno == EINTR && !EINTR_Valid);

	return r;
}

